@@ -27,6 +27,7 @@
 #include "eog-list-store.h"
 #include "eog-image.h"
 #include "eog-job-scheduler.h"
+#include "eog-util.h"
 
 #ifdef HAVE_EXIF
 #include "eog-exif-util.h"
@@ -82,6 +83,7 @@ struct _EogThumbViewPrivate {
 	gint n_images;
 	gulong image_add_id;
 	gulong image_removed_id;
+	gulong image_thumbnail_id;
 
 	gboolean indices_changed;
 };
@@ -179,6 +181,11 @@ eog_thumb_view_dispose (GObject *object)
 		priv->image_removed_id = 0;
 	}
 
+	if (model && priv->image_thumbnail_id) {
+		g_signal_handler_disconnect (model, priv->image_thumbnail_id);
+		priv->image_thumbnail_id = 0;
+	}
+
 	G_OBJECT_CLASS (eog_thumb_view_parent_class)->dispose (object);
 }
 
@@ -513,15 +520,11 @@ thumbview_get_tooltip_string (EogImage *image)
 
 	if (width > -1 && height > -1) {
 		tooltip_string = g_markup_printf_escaped ("<b><big>%s</big></b>\n"
-							  "%i x %i %s\n"
+							  "%s\n"
 							  "%s\n"
 							  "%s",
 							  eog_image_get_caption (image),
-							  width,
-							  height,
-							  ngettext ("pixel",
-								    "pixels",
-								    height),
+							  eog_util_create_width_height_string(width, height),
 							  bytes,
 							  type_str);
 	} else {
@@ -645,6 +648,7 @@ eog_thumb_view_init (EogThumbView *thumbview)
 	thumbview->priv->visible_range_changed_id = 0;
 	thumbview->priv->image_add_id = 0;
 	thumbview->priv->image_removed_id = 0;
+	thumbview->priv->image_thumbnail_id = 0;
 }
 
 /**
@@ -703,6 +707,29 @@ eog_thumb_view_row_deleted_cb (GtkTreeModel    *tree_model,
 	eog_thumb_view_update_columns (view);
 }
 
+static void
+eog_thumb_view_row_changed_cb (GtkTreeModel *model,
+			       GtkTreePath  *path,
+			       GtkTreeIter  *iter,
+			       gpointer      data)
+{
+
+	guint signal_id;
+	signal_id = GPOINTER_TO_UINT (data);
+	/* PREVENT GtkIconView "row-changed" handler to be reached, as it will
+	 * perform a full invalidate and relayout of all items, See bug:
+	 * https://bugzilla.gnome.org/show_bug.cgi?id=691448#c9 */
+	g_signal_stop_emission (model, signal_id, 0);
+}
+
+static void
+eog_thumb_view_draw_thumbnail_cb (EogListStore *store,
+				  EogThumbView *view)
+{
+        gtk_widget_queue_draw (GTK_WIDGET(view));
+	return;
+}
+
 /**
  * eog_thumb_view_set_model:
  * @thumbview: A #EogThumbView.
@@ -736,8 +763,18 @@ eog_thumb_view_set_model (EogThumbView *thumbview, EogListStore *store)
 			                             priv->image_removed_id);
 
 		}
+		if (priv->image_thumbnail_id != 0) {
+			g_signal_handler_disconnect (existing,
+			                             priv->image_thumbnail_id);
+
+		}
 	}
 
+	guint signal_id = g_signal_lookup ("row-changed", GTK_TYPE_TREE_MODEL);
+	g_signal_connect (GTK_TREE_MODEL (store), "row-changed",
+			  G_CALLBACK (eog_thumb_view_row_changed_cb),
+			  GUINT_TO_POINTER (signal_id));
+
 	priv->image_add_id = g_signal_connect (G_OBJECT (store), "row-inserted",
 	                            G_CALLBACK (eog_thumb_view_row_inserted_cb),
 	                            thumbview);
@@ -745,6 +782,10 @@ eog_thumb_view_set_model (EogThumbView *thumbview, EogListStore *store)
 	                             "row-deleted",
 	                             G_CALLBACK (eog_thumb_view_row_deleted_cb),
 	                             thumbview);
+	priv->image_thumbnail_id = g_signal_connect (G_OBJECT (store),
+	                             "draw-thumbnail",
+	                             G_CALLBACK (eog_thumb_view_draw_thumbnail_cb),
+				     thumbview);
 
 	thumbview->priv->start_thumb = thumbview->priv->end_thumb = 0;
 	thumbview->priv->n_images = eog_list_store_length (store);
@@ -967,6 +1008,10 @@ eog_thumb_view_select_single (EogThumbView *thumbview,
 
 	model = gtk_icon_view_get_model (GTK_ICON_VIEW (thumbview));
 
+	if (!model) {
+		return;
+	}
+
 	n_items = eog_list_store_length (EOG_LIST_STORE (model));
 
 	if (n_items == 0) {
